@@ -382,7 +382,7 @@ case `uname -m` in
     9000/800) guessed_sbcl_arch=hppa ;;
     mips*) guessed_sbcl_arch=mips ;;
     *arm*) guessed_sbcl_arch=arm ;;
-    aarch64) guessed_sbcl_arch=arm ;;
+    aarch64) guessed_sbcl_arch=arm64 ;;
     *)
         # If we're not building on a supported target architecture, we
         # we have no guess, but it's not an error yet, since maybe
@@ -733,6 +733,13 @@ elif [ "$sbcl_arch" = "arm" ]; then
     printf ' :stack-allocatable-vectors :stack-allocatable-closures' >> $ltf
     printf ' :precise-arg-count-error :unwind-to-frame-and-call-vop' >> $ltf
     printf ' :fp-and-pc-standard-save' >> $ltf
+elif [ "$sbcl_arch" = "arm64" ]; then
+    printf ' :64-bit :gencgc :linkage-table :fp-and-pc-standard-save' >> $ltf
+    printf ' :alien-callbacks :precise-arg-count-error' >> $ltf
+    printf ' :ash-right-vops :multiply-high-vops :symbol-info-vops' >> $ltf
+    printf ' :stack-allocatable-lists :stack-allocatable-fixed-objects' >> $ltf
+    printf ' :stack-allocatable-vectors :stack-allocatable-closures' >> $ltf
+    printf ' :precise-arg-count-error :unwind-to-frame-and-call-vop' >> $ltf
 else
     # Nothing need be done in this case, but sh syntax wants a placeholder.
     echo > /dev/null
