@@ -116,7 +116,7 @@
       function
       (cond ((functionp x) x)
             ((null x) ,fallback)
-            (t (values (sb!sys:%primitive sb!c:safe-fdefn-fun x)))))))
+            (t (sb!c:safe-fdefn-fun x))))))
 
 ;; Return a function-designator given a character-macro-table entry.
 (defmacro !cmt-entry-to-fun-designator (val)
@@ -676,16 +676,34 @@ standard Lisp readtable when NIL."
   (declare (optimize (sb!c::check-tag-existence 0)))
   (if recursive-p
       ;; a loop for repeating when a macro returns nothing
-      (loop
-       (let ((char (read-char stream eof-error-p +EOF+)))
-         (cond ((eq char +EOF+) (return eof-value))
-               ((whitespace[2]p char))
-               (t
-                (multiple-value-bind (result-p result)
-                    (read-maybe-nothing stream char)
-                  ;; Repeat if macro returned nothing.
-                  (unless (zerop result-p)
-                    (return (unless *read-suppress* result))))))))
+      (let* ((tracking-p (form-tracking-stream-p stream))
+             (outermost-p
+              (and tracking-p
+                   (null (form-tracking-stream-form-start-char-pos stream)))))
+        (loop
+         (let ((char (read-char stream eof-error-p +EOF+)))
+           (cond ((eq char +EOF+) (return eof-value))
+                 ((whitespace[2]p char))
+                 (t
+                  (when outermost-p
+                    ;; Calling FILE-POSITION at each token seems to slow down
+                    ;; the reader by somewhere between 8x to 10x.
+                    ;; Once per outermost form is acceptably fast though.
+                    (setf (form-tracking-stream-form-start-byte-pos stream)
+                          ;; pretend we queried the position before reading CHAR
+                          (- (file-position stream)
+                             (or (file-string-length stream (string char)) 0))
+                          (form-tracking-stream-form-start-char-pos stream)
+                          ;; likewise
+                          (1- (form-tracking-stream-input-char-pos stream))))
+                  (multiple-value-bind (result-p result)
+                      (read-maybe-nothing stream char)
+                    (unless (zerop result-p)
+                      (return (unless *read-suppress* result)))
+                    ;; Repeat if macro returned nothing.
+                    (when tracking-p
+                      (funcall (form-tracking-stream-observer stream)
+                               :reset nil nil))))))))
       (let ((*sharp-equal-alist* nil))
         (with-read-buffer ()
           (%read-preserving-whitespace stream eof-error-p eof-value t)))))
@@ -708,13 +726,24 @@ standard Lisp readtable when NIL."
 ;;; for functions that want comments to return so that they can look
 ;;; past them. CHAR must not be whitespace.
 (defun read-maybe-nothing (stream char)
-  (multiple-value-call
-   (lambda (&optional (result nil supplied-p) &rest junk)
-     (declare (ignore junk)) ; is this ANSI-specified?
-     (values (if supplied-p 1 0) result))
-   (funcall (!cmt-entry-to-function
-             (get-raw-cmt-entry char *readtable*) #'read-token)
-            stream char)))
+  (truly-the
+   (values bit t) ; avoid a type-check. M-V-CALL is lame
+   (multiple-value-call
+       (lambda (stream start-pos &optional (result nil supplied-p) &rest junk)
+         (declare (ignore junk)) ; is this ANSI-specified?
+         (when (and supplied-p start-pos)
+           (funcall (form-tracking-stream-observer stream)
+                    start-pos
+                    (form-tracking-stream-input-char-pos stream) result))
+         (values (if supplied-p 1 0) result))
+     ;; KLUDGE: not capturing anything in the lambda avoids closure consing
+     stream
+     (and (form-tracking-stream-p stream)
+          ;; Subtract 1 because the position points _after_ CHAR.
+          (1- (form-tracking-stream-input-char-pos stream)))
+     (funcall (!cmt-entry-to-function
+               (get-raw-cmt-entry char *readtable*) #'read-token)
+              stream char))))
 
 (defun read (&optional (stream *standard-input*)
                        (eof-error-p t)
@@ -769,6 +798,11 @@ standard Lisp readtable when NIL."
   ;; Don't return anything.
   (values))
 
+;;; FIXME: for these two macro chars, if STREAM is a FORM-TRACKING-STREAM,
+;;; every cons cell should generate a notification so that the readtable
+;;; manipulation in SB-COVER can be eliminated in favor of a stream observer.
+;;; It is cheap to add events- it won't increase consing in the compiler
+;;; because it the extra events can simply be ignored.
 (macrolet
     ((with-list-reader ((streamvar delimiter) &body body)
        `(let* ((thelist (list nil))
@@ -860,7 +894,9 @@ standard Lisp readtable when NIL."
   (declare (character closech))
   (let ((stream (in-synonym-of stream))
         (buf *read-buffer*)
-        (rt *readtable*))
+        (rt *readtable*)
+        ;; *read-suppress* => "... macros will not construct any new objects"
+        (suppress *read-suppress*))
     (reset-read-buffer buf)
     (macrolet ((scan (read-a-char eofp &optional finish)
                  `(loop (let ((char ,read-a-char))
@@ -871,13 +907,15 @@ standard Lisp readtable when NIL."
                                  (setq char ,read-a-char)
                                  (when ,eofp
                                    (error 'end-of-file :stream stream))))
-                         (ouch-read-buffer (truly-the character char) buf)))))
+                          (unless suppress
+                            (ouch-read-buffer (truly-the character char)
+                                              buf))))))
       (if (ansi-stream-p stream)
           (prepare-for-fast-read-char stream
            (scan (fast-read-char t) nil (done-with-fast-read-char)))
         ;; CLOS stream
           (scan (read-char stream nil +EOF+) (eq char +EOF+))))
-    (copy-token-buf-string buf)))
+    (if suppress "" (copy-token-buf-string buf))))
 
 (defun read-right-paren (stream ignore)
   (declare (ignore ignore))
@@ -1474,10 +1512,10 @@ extended <package-name>::<form-in-package> syntax."
                      (reader-find-package package-designator stream)
                      (or *reader-package* (sane-package)))))
         (if (or (zerop colons) (= colons 2) (eq pkg *keyword-package*))
-            (return (intern* (token-buf-string buf) (token-buf-fill-ptr buf)
-                             pkg))
+            (return (%intern (token-buf-string buf) (token-buf-fill-ptr buf)
+                             pkg t))
             (multiple-value-bind (symbol accessibility)
-                (find-symbol* (token-buf-string buf) (token-buf-fill-ptr buf)
+                (%find-symbol (token-buf-string buf) (token-buf-fill-ptr buf)
                               pkg)
               (when (eq accessibility :external) (return symbol))
               (let ((name (copy-token-buf-string buf)))
