@@ -16,6 +16,23 @@
 
 (in-package "SB!IMPL")
 
+;;; Return T if FUN names a DEFSTRUCT slot accessor that we should
+;;; transform from SETF into %INSTANCE-SET directly - bypassing
+;;; #'(SETF MYSLOT) - which requires that the slot be read/writable.
+;;; A local function named (SETF MYSLOT) inhibits the transform,
+;;; though technically need not, as it is unspecified how SETF
+;;; of a structure slot expands. It is likewise unportable to
+;;; expect that a NOTINLINE does anything, but we'll check anyway.
+(defun transformable-struct-setf-p (form env)
+  (when (singleton-p (cdr form))
+    (let* ((fun (car form))
+           (slot-info (structure-instance-accessor-p fun)))
+      (when (and slot-info (not (dsd-read-only (cdr slot-info))))
+        (dx-let ((setter `(setf ,fun)))
+          (when (and (not (sb!c::fun-locally-defined-p setter env))
+                     (not (sb!c::fun-lexically-notinline-p setter env)))
+            slot-info)))))) ; caller needs the (DD . DSD) pair pack
+
 ;;; The inverse for a generalized-variable reference function is stored in
 ;;; one of two ways:
 ;;;
@@ -30,7 +47,10 @@
 ;;; and an accessing function.
 (declaim (ftype (function (t &optional lexenv-designator))
                 sb!xc:get-setf-expansion))
-(defun sb!xc:get-setf-expansion (form &optional environment)
+(defun sb!xc:get-setf-expansion (form &optional environment
+                                      ;; Assume we'll need one store temp.
+                                      ;; That's the expected thing.
+                                      &aux (store (sb!xc:gensym "NEW")))
   #!+sb-doc
   "Return five values needed by the SETF machinery: a list of temporary
    variables, a list of values with which to fill them, a list of temporaries
@@ -40,36 +60,44 @@
           (sb!xc:macroexpand-1 form environment)
         (if expanded
             (sb!xc:get-setf-expansion expansion environment)
-            (let ((store (sb!xc:gensym "NEW")))
-              (values nil nil (list store) `(setq ,form ,store) form))))
-      (let ((name (car form)))
+            (values nil nil (list store) `(setq ,form ,store) form)))
+      (let ((fun (car form)))
         (flet ((expand (call arg-maker)
                  ;; Produce the expansion of a SETF form that calls either
                  ;; #'(SETF name) or an inverse given by short form DEFSETF.
                  (multiple-value-bind (temp-vars temp-vals args)
                      (collect-setf-temps (cdr form) environment nil)
-                   (let ((store (sb!xc:gensym "NEW")))
-                     (values temp-vars temp-vals (list store)
-                             `(,.call ,@(funcall arg-maker store args))
-                             `(,name ,@args))))))
+                   (values temp-vars temp-vals (list store)
+                           `(,.call ,@(funcall arg-maker store args))
+                           `(,fun ,@args)))))
           ;; Local functions inhibit global SETF methods.
-          (unless (sb!c::fun-locally-defined-p name environment)
-            (acond ((info :setf :inverse name)
+          (unless (sb!c::fun-locally-defined-p fun environment)
+            (acond ((info :setf :inverse fun)
                     (return-from sb!xc:get-setf-expansion
                       (expand `(,it) (lambda (new args) `(,@args ,new)))))
-                   ((info :setf :expander name)
+                   ((info :setf :expander fun)
                     (return-from sb!xc:get-setf-expansion
                       (if (consp it)
                           (make-setf-quintuple form environment
                                                (car it) (cdr it))
-                          (funcall it form environment))))))
+                          (funcall it form environment))))
+                   ((transformable-struct-setf-p form environment)
+                    (let ((instance (make-symbol "OBJ")))
+                      (return-from sb!xc:get-setf-expansion
+                        (values (list instance)
+                                (list (cadr form))
+                                (list store)
+                                (slot-access-transform
+                                 :setf (list instance store) it)
+                                (slot-access-transform
+                                 :read (list instance) it)))))))
           ;; When NAME is a macro, retry from the top.
           ;; Otherwise default to the function named `(SETF ,name).
           (multiple-value-bind (expansion expanded)
               (%macroexpand-1 form environment)
             (if expanded
                 (sb!xc:get-setf-expansion expansion environment)
-                (expand `(funcall #'(setf ,name)) #'cons)))))))
+                (expand `(funcall #'(setf ,fun)) #'cons)))))))
 
 ;; Expand PLACE until it is a form that SETF might know something about.
 ;; Macros are expanded only when no SETF expander (or inverse) exists.
@@ -134,10 +162,18 @@
         (return-from setf `(progn ,@(sb!c::explode-setq form 'error))))
       (when (symbolp (setq place (macroexpand-for-setf place env)))
         (return-from setf `(setq ,place ,value-form)))
-      (let* ((fun (car place))
-             (inverse (info :setf :inverse fun)))
-        (when (and inverse (not (sb!c::fun-locally-defined-p fun env)))
-          (return-from setf `(,inverse ,@(cdr place) ,value-form))))
+
+      (let ((fun (car place)))
+        (when (and (symbolp fun)
+                   ;; Local definition of FUN precludes global knowledge.
+                   (not (sb!c::fun-locally-defined-p fun env)))
+          (awhen (info :setf :inverse fun)
+            (return-from setf `(,it ,@(cdr place) ,value-form)))
+          (awhen (transformable-struct-setf-p place env)
+            (return-from setf
+              (slot-access-transform
+               :setf (list (cadr place) value-form) it)))))
+
       (multiple-value-bind (temps vals newval setter)
           (sb!xc:get-setf-expansion place env)
         (car (gen-let* (mapcar #'list temps vals)
@@ -151,7 +187,6 @@
    value expression. Evaluates all of the expressions in turn, then
    assigns the value of each expression to the place on its left,
    returning the value of the leftmost."
-  (declare (type sb!c::lexenv env))
   (when (< (length args) 2)
     (error "~S called with too few arguments: ~S" 'shiftf form))
   (collect ((let-bindings) (mv-bindings) (setters) (getters))
@@ -226,6 +261,7 @@
    forms have been evaluated."
     (expand pairs env 'psetq 'setq))))
 
+;;; FIXME: the following claim could not possibly be true, could it?
 ;;; FIXME: Compiling this definition of ROTATEF apparently blows away the
 ;;; definition in the cross-compiler itself, so that after that, any
 ;;; ROTATEF operations can no longer be compiled, because
@@ -236,7 +272,6 @@
    expressions in turn, then assigns to each place the value of the form to
    its right. The rightmost form gets the value of the leftmost.
    Returns NIL."
-  (declare (type sb!c::lexenv env))
   (when args
     (collect ((let*-bindings) (mv-bindings) (setters) (getters))
       (dolist (arg args)
@@ -587,7 +622,6 @@
        (%defsetf ',access-fn ,def nil ,@(and doc `(,doc))))))
 
 (sb!xc:define-setf-expander values (&rest places &environment env)
-  (declare (type sb!c::lexenv env))
   (collect ((setters) (getters))
     (let ((all-dummies '())
           (all-vals '())
@@ -607,7 +641,6 @@
               `(values ,@(setters)) `(values ,@(getters))))))
 
 (sb!xc:define-setf-expander getf (place prop &optional default &environment env)
-  (declare (type sb!c::lexenv env))
   (binding* (((place-tempvars place-tempvals stores set get)
               (sb!xc:get-setf-expansion place env))
              ((call-tempvars call-tempvals call-args bitmask)
@@ -740,7 +773,6 @@ with bits from the corresponding position in the new value."
   (setf-expand-ldb bytespec place env 'deposit-field 'mask-field))
 
 (defun setf-expand-the (the type place env)
-  (declare (type sb!c::lexenv env))
   (multiple-value-bind (temps subforms store-vars setter getter)
       (sb!xc:get-setf-expansion place env)
     (values temps subforms store-vars
