@@ -32,12 +32,6 @@
                   *interrupt-pending*
                   #!+sb-thruption *thruption-pending*))
 
-(!defglobal *cold-init-complete-p* nil)
-
-;;; counts of nested errors (with internal errors double-counted)
-(!defvar *maximum-error-depth* 10)
-(!defvar *current-error-depth* 0)
-
 ;;;; default initfiles
 
 (defun sysinit-pathname ()
@@ -121,46 +115,6 @@ means to wait indefinitely.")
                  (setf ok t))
             (os-exit code :abort (not ok)))))))
 
-;;;; working with *CURRENT-ERROR-DEPTH* and *MAXIMUM-ERROR-DEPTH*
-
-;;; INFINITE-ERROR-PROTECT is used by ERROR and friends to keep us out
-;;; of hyperspace.
-(defmacro infinite-error-protect (&rest forms)
-  `(unless (infinite-error-protector)
-     (/show0 "back from INFINITE-ERROR-PROTECTOR")
-     (let ((*current-error-depth* (1+ *current-error-depth*)))
-       (/show0 "in INFINITE-ERROR-PROTECT, incremented error depth")
-       ;; arbitrary truncation
-       #!+sb-show (sb!debug:print-backtrace :count 8)
-       ,@forms)))
-
-;;; a helper function for INFINITE-ERROR-PROTECT
-(defun infinite-error-protector ()
-  (/show0 "entering INFINITE-ERROR-PROTECTOR, *CURRENT-ERROR-DEPTH*=..")
-  (/hexstr *current-error-depth*)
-  (unless *cold-init-complete-p*
-    (%primitive print "Argh! error in cold init, halting")
-    (%primitive sb!c:halt))
-  ;; Checking BOUNDP on these variables would be superfluous
-  ;; because REALP will return false for unbound-marker.
-  (let ((cur (locally (declare (optimize (safety 0))) *current-error-depth*))
-        (max (locally (declare (optimize (safety 0))) *maximum-error-depth*)))
-    (cond ((or (not (realp cur)) (not (realp max))) ; why not just FIXNUMP?
-           (%primitive print "Argh! corrupted error depth, halting")
-           (%primitive sb!c:halt))
-          ((> cur max)
-           (/show0 "*MAXIMUM-ERROR-DEPTH*=..")
-           (/hexstr max)
-           (/show0 "in INFINITE-ERROR-PROTECTOR, calling ERROR-ERROR")
-           (error-error "Help! "
-                        cur
-                        " nested errors. "
-                        "SB-KERNEL:*MAXIMUM-ERROR-DEPTH* exceeded.")
-           t)
-          (t
-           (/show0 "returning normally from INFINITE-ERROR-PROTECTOR")
-           nil))))
-
 ;;;; miscellaneous external functions
 
 (defun split-seconds-for-sleep (seconds)
@@ -276,6 +230,11 @@ any non-negative real number."
       ;; 2. Rebind the stream symbol in case some poor sod sees
       ;;    a broken stream here while running with *BREAK-ON-ERRORS*.
       (let ((stream (stream-output-stream (symbol-value name))))
+        ;; This is kind of crummy because it checks in globaldb for each
+        ;; stream symbol whether it can be bound to a stream. The translator
+        ;; for PROGV could skip ABOUT-TO-MODIFY-SYMBOL-VALUE based on
+        ;; an aspect of a policy, but if users figure that out they could
+        ;; do something horrible like rebind T and NIL.
         (progv (list name) (list null)
           (handler-bind ((stream-error
                            (lambda (c)
