@@ -11,96 +11,16 @@
 
 (in-package "SB!C")
 
-(defvar *handled-conditions* nil)
-(defvar *disabled-package-locks* nil)
-
-;;; The LEXENV represents the lexical environment used for IR1 conversion.
-;;; (This is also what shows up as an ENVIRONMENT value in macroexpansion.)
-#!-sb-fluid (declaim (inline internal-make-lexenv)) ; only called in one place
-(def!struct (lexenv
-             (:print-function print-lexenv)
-             (:constructor make-null-lexenv ())
-             (:constructor internal-make-lexenv
-                           (funs vars blocks tags
-                            type-restrictions
-                            lambda cleanup handled-conditions
-                            disabled-package-locks %policy user-data
-                            parent)))
-  ;; an alist of (NAME . WHAT), where WHAT is either a FUNCTIONAL (a
-  ;; local function), a DEFINED-FUN, representing an
-  ;; INLINE/NOTINLINE declaration, or a list (MACRO . <function>) (a
-  ;; local macro, with the specifier expander). Note that NAME may be
-  ;; a (SETF <name>) list, not necessarily a single symbol.
-  (funs nil :type list)
-  ;; an alist translating variable names to LEAF structures. A special
-  ;; binding is indicated by a :SPECIAL GLOBAL-VAR leaf. Each special
-  ;; binding within the code gets a distinct leaf structure, as does
-  ;; the current "global" value on entry to the code compiled.
-  ;; (locally (special ...)) is handled by adding the most recent
-  ;; special binding to the front of the list.
-  ;;
-  ;; If the CDR is (MACRO . <exp>), then <exp> is the expansion of a
-  ;; symbol macro.
-  (vars nil :type list)
-  ;; BLOCKS and TAGS are alists from block and go-tag names to 2-lists
-  ;; of the form (<entry> <continuation>), where <continuation> is the
-  ;; continuation to exit to, and <entry> is the corresponding ENTRY
-  ;; node.
-  (blocks nil :type list)
-  (tags nil :type list)
-  ;; an alist (THING . CTYPE) which is used to keep track of
-  ;; "pervasive" type declarations. When THING is a leaf, this is for
-  ;; type declarations that pertain to the type in a syntactic extent
-  ;; which does not correspond to a binding of the affected name.
-  (type-restrictions nil :type list)
-  ;; the lexically enclosing lambda, if any
-  ;;
-  ;; FIXME: This should be :TYPE (OR CLAMBDA NULL), but it was too hard
-  ;; to get CLAMBDA defined in time for the cross-compiler.
-  (lambda nil)
-  ;; the lexically enclosing cleanup, or NIL if none enclosing within LAMBDA
-  (cleanup nil)
-  ;; condition types we handle with a handler around the compiler
-  (handled-conditions *handled-conditions*)
-  ;; lexically disabled package locks (list of symbols)
-  (disabled-package-locks *disabled-package-locks*)
-  ;; the current OPTIMIZE policy. this is null in the null environment,
-  ;; and the global policy is stored in *POLICY*. (Because we want to
-  ;; be able to affect it from :WITH-COMPILATION-UNIT.) NIL here also
-  ;; works as a convenient null-lexenv identifier.
-  (%policy nil :type (or null policy))
-  ;; A list associating extra user info to symbols.  The entries
-  ;; are of the form (:declare name . value),
-  ;; (:variable name key . value), or (:function name key . value)
-  (user-data nil :type list)
-  parent)
-
-;;; the lexical environment we are currently converting in
-(defvar *lexenv*)
-(declaim (type lexenv *lexenv*))
-
-;;; an object suitable for input to standard functions that accept
-;;; "environment objects" (of the ANSI glossary)
-(def!type lexenv-designator () '(or lexenv null))
-
-(defun lexenv-policy (lexenv)
-  (or (lexenv-%policy lexenv) *policy*))
-
-(defun null-lexenv-p (lexenv)
-  (not (lexenv-%policy lexenv)))
-
 ;;; support for the idiom (in MACROEXPAND and elsewhere) that NIL is
-;;; to be taken as a null lexical environment
+;;; to be taken as a null lexical environment.
+;;; Of course this is a mostly pointless "idiom" because NIL *is*
+;;; an environment, as far as most environment inquiry functions care.
 (defun coerce-to-lexenv (x)
   (etypecase x
     (null (make-null-lexenv))
-    (lexenv x)))
-
-(defun print-lexenv (lexenv stream level)
-  (if (null-lexenv-p lexenv)
-      (print-unreadable-object (lexenv stream)
-        (write-string "NULL-LEXENV" stream))
-      (default-structure-print lexenv stream level)))
+    (lexenv x)
+    #!+sb-fasteval
+    (sb!interpreter:basic-env (sb!interpreter:lexenv-from-env x))))
 
 ;;; Take the lexenv surrounding an inlined function and extract things
 ;;; needed for the inline expansion suitable for dumping into fasls.
@@ -174,8 +94,11 @@
                      (not (null-lexenv-p parent))))
     result))
 
+;;; Return a sexpr for LAMBDA in LEXENV such that loading it from fasl
+;;; preserves the original lexical environment for inlining.
+;;; Return NIL if the lexical environment is too complicated.
 (defun maybe-inline-syntactic-closure (lambda lexenv)
-  (declare (type list lambda) (type lexenv lexenv))
+  (declare (type list lambda) (type lexenv-designator lexenv))
   (aver (eql (first lambda) 'lambda))
   ;; We used to have a trivial implementation, verifying that lexenv
   ;; was effectively null. However, this fails to take account of the
@@ -188,15 +111,21 @@
   ;; which, while too complicated for the cross-compiler to handle in
   ;; unfriendly foreign lisp environments, would be good to support in
   ;; the target compiler. -- CSR, 2002-05-13 and 2002-11-02
-  (let ((vars (lexenv-vars lexenv))
-        (funs (lexenv-funs lexenv)))
-    (cond
-      ((or (lexenv-blocks lexenv) (lexenv-tags lexenv)) nil)
-      ((and (null vars) (null funs)) lambda)
-      ;; too complicated for cross-compilation
-      #-sb-xc-host
-      (t
-       (let ((env (reconstruct-lexenv lexenv)))
-         (and env
-          `(lambda-with-lexenv ,env ,@(cdr lambda))))))))
-
+  (typecase lexenv
+   (lexenv
+    (let ((vars (lexenv-vars lexenv))
+          (funs (lexenv-funs lexenv)))
+      (cond ((or (lexenv-blocks lexenv) (lexenv-tags lexenv)) nil)
+            ((and (null vars) (null funs)) lambda)
+            ;; too complicated for cross-compilation
+            #-sb-xc-host
+            (t
+             (let ((env (reconstruct-lexenv lexenv)))
+               (and env
+                    `(lambda-with-lexenv ,env ,@(cdr lambda))))))))
+   #!+sb-fasteval
+   (sb!interpreter:basic-env
+    (awhen (sb!interpreter::reconstruct-syntactic-closure-env lexenv)
+      `(lambda-with-lexenv ,it ,@(cdr lambda))))
+   #!+sb-fasteval
+   (null lambda))) ; trivial case. Never occurs in the compiler.
