@@ -20,6 +20,19 @@
 ;;; blocks -- with BLOCK-SUCC/BLOCK-PRED lists; data transfers are
 ;;; represented with LVARs.
 
+;;; FIXME: this file contains a ton of DEF!STRUCT definitions most of which
+;;; could be DEFSTRUCT, except for the fact that we use def!struct as
+;;; a workaround for the compiler's inability to cope with mutally referential
+;;; structures, even ones within the same file. The IR1 structures are tightly
+;;; knitted together - for example, starting from a CRETURN, you can reach
+;;; at least 15 other structure objects, not counting some like HASH-TABLE
+;;; which are fundamental. e.g. we have:
+;;;  CRETURN -> {NODE,CTRAN,CLAMBDA,LVAR}
+;;;  CTRAN -> {BLOCK},
+;;;  CLAMBDA -> {FUNCTIONAL,COMBINATION,BIND,PHYSENV,OPTIONAL-DISPATCH}
+;;; and so on.  DEF!STRUCT solves this problem by way of a terrible hack
+;;; that works only for compiling the compiler.
+
 ;;; "Lead-in" Control TRANsfer [to some node]
 (def!struct (ctran
              (:make-load-form-fun ignore-it)
@@ -192,10 +205,10 @@
   (defattr block-type-asserted)
   (defattr block-test-modified))
 
-(defstruct (cloop (:conc-name loop-)
-                  (:predicate loop-p)
-                  (:constructor make-loop)
-                  (:copier copy-loop))
+(def!struct (cloop (:conc-name loop-)
+                   (:predicate loop-p)
+                   (:constructor make-loop)
+                   (:copier copy-loop))
   ;; The kind of loop that this is.  These values are legal:
   ;;
   ;;    :OUTER
@@ -334,6 +347,14 @@
 ;;;   size of flow analysis problems, this allows back-end data
 ;;;   structures to be reclaimed after the compilation of each
 ;;;   component.
+(locally
+  ;; This is really taking the low road. I couldn't think of a way to
+  ;; avoid a style warning regarding IR2-COMPONENT other than to declare
+  ;; the INFO slot as :type (or (satisfies ir2-component-p) ...)
+  ;; During make-host-2, the solution to this is the same hack
+  ;; as for everything else: use DEF!STRUCT for IR2-COMPONENT.
+  #!+(and (host-feature sb-xc-host) (host-feature sbcl))
+  (declare (sb-ext:muffle-conditions style-warning))
 (def!struct (component (:copier nil)
                        (:constructor
                         make-component
@@ -444,7 +465,7 @@
   ;; The default LOOP in the component.
   (outer-loop (missing-arg) :type cloop)
   ;; The current sset index
-  (sset-number 0 :type fixnum))
+  (sset-number 0 :type fixnum)))
 (defprinter (component :identity t)
   name
   #!+sb-show id
@@ -458,6 +479,7 @@
   ;; COMPILE-COMPONENT hasn't happened yet. Might it be even better
   ;; (certainly stricter, possibly also correct...) to assert that
   ;; IR1-FINALIZE hasn't happened yet?
+  #+sb-xc-host (declare (notinline component-info)) ; unknown type
   (aver (not (eql (component-info component) :dead))))
 
 ;;; A CLEANUP structure represents some dynamic binding action. Blocks
@@ -684,22 +706,6 @@
   (aver (leaf-has-source-name-p leaf))
   (leaf-%source-name leaf))
 
-;;; The CONSTANT structure is used to represent known constant values.
-;;; Since the same constant leaf may be shared between named and anonymous
-;;; constants, %SOURCE-NAME is never used.
-(def!struct (constant (:constructor make-constant (value
-                                                   &aux
-                                                   (type (ctype-of value))
-                                                   (%source-name '.anonymous.)
-                                                   (where-from :defined)))
-                      (:include leaf))
-  ;; the value of the constant
-  (value (missing-arg) :type t)
-  ;; Boxed TN for this constant, if any.
-  (boxed-tn nil :type (or null tn)))
-(defprinter (constant :identity t)
-  value)
-
 ;;; The BASIC-VAR structure represents information common to all
 ;;; variables which don't correspond to known local functions.
 (def!struct (basic-var (:include leaf)
@@ -720,10 +726,16 @@
   (defined-type :test (not (eq defined-type *universal-type*)))
   (where-from :test (not (eq where-from :assumed)))
   kind)
+
 (defun fun-locally-defined-p (name env)
-  (and env
-       (let ((fun (cdr (assoc name (lexenv-funs env) :test #'equal))))
-         (and fun (not (global-var-p fun))))))
+  (typecase env
+    (null nil)
+    #!+sb-fasteval
+    (sb!interpreter:basic-env
+     (values (sb!interpreter:find-lexical-fun env name)))
+    (t
+     (let ((fun (cdr (assoc name (lexenv-funs env) :test #'equal))))
+       (and fun (not (global-var-p fun)))))))
 
 ;;; A DEFINED-FUN represents a function that is defined in the same
 ;;; compilation block, or that has an inline expansion, or that has a
@@ -1462,16 +1474,30 @@
 ;;; but no, NIL was always an empty alist representing no qualities,
 ;;; which is a valid policy that makes each quality read as 1.
 ;;; In contrast, a LEXENV with NIL policy _does_ become *POLICY*.
+;;; The reason for NIL mapping to baseline is that all nodes are annotated
+;;; with a LEXENV, and the only object type that can be a LEXENV is LEXENV.
+;;; An indicator is needed that a LEXENV is devoid of a policy, so this is
+;;; what the NIL is for in lexenv-policy. But sometimes the compiler needs
+;;; a policy without reference to an IR object - which is weird - and in that
+;;; case it has nothing better to go with but the baseline policy.
+;;; It still seems like a bug though.
 (defun %coerce-to-policy (thing)
-  (cond ((policy-p thing) thing)
-        (thing (lexenv-policy (etypecase thing
-                                (lexenv thing)
-                                (node (node-lexenv thing))
-                                (functional (functional-lexenv thing)))))
-        (t **baseline-policy**)))
+  (typecase thing
+    (policy thing)
+    #!+sb-fasteval
+    (sb!interpreter:basic-env (sb!interpreter:env-policy thing))
+    (null **baseline-policy**)
+    (t (lexenv-policy (etypecase thing
+                        (lexenv thing)
+                        (node (node-lexenv thing))
+                        (functional (functional-lexenv thing)))))))
 
 ;;;; Freeze some structure types to speed type testing.
 
+;; FIXME: the frozen-ness can't actually help optimize anything
+;; until this file is compiled by the cross-compiler.
+;; Anything compiled prior to then uses the non-frozen classoid as existed
+;; at load-time of the cross-compiler. SB!XC:PROCLAIM would likely work here.
 #!-sb-fluid
-(declaim (freeze-type node leaf lexenv ctran lvar cblock component cleanup
+(declaim (freeze-type node lexenv ctran lvar cblock component cleanup
                       physenv tail-set nlx-info))
