@@ -81,7 +81,7 @@
 ;;; and that it contains the frames we expect, doesn't contain any
 ;;; "bogus stack frame"s, and contains the appropriate toplevel call
 ;;; and hasn't been cut off anywhere.
-(defun verify-backtrace (test-function frame-specs &key (allow-stunted nil) details)
+(defun verify-backtrace (test-function frame-specs &key details)
   (labels ((args-equal (want real)
              (cond ((eq '&rest (car want))
                     t)
@@ -198,13 +198,7 @@
                    (list `(flet test :in ,*p*) #'not-optimized))))))
 
 (with-test (:name :backtrace-interrupted-condition-wait
-            :skipped-on '(not :sb-thread)
-                  ;; For some unfathomable reason the backtrace becomes
-                  ;; stunted, ending at _sigtramp, when we add :TIMEOUT NIL to
-                  ;; the frame we expect. If we leave it out, the backtrace is
-                  ;; fine -- but the test fails. I can only boggle right now.
-            :fails-on `(or (and :x86 :linux)
-                           :win32))
+            :skipped-on '(not :sb-thread))
   (let ((m (sb-thread:make-mutex))
         (q (sb-thread:make-waitqueue)))
     (assert (verify-backtrace
@@ -289,7 +283,7 @@
                                '((,two-arg 42 t)
                                  #+(or x86 x86-64)
                                  ,@(and predicate
-                                    '(("no debug information for frame")))
+                                    `((,(find-symbol (format nil "GENERIC-~A" fun) "SB-VM"))))
                                  ((flet ,test-name :in ,*p*) 42 t))))))))
              (test-predicates (&rest functions)
                `(progn ,@(mapcar (lambda (function)
@@ -858,7 +852,8 @@
    (block nil
      (handler-bind ((error (lambda (e)
                              (declare (ignore e))
-                             (return (< (length (car (sb-debug:backtrace-as-list 1))) 10)))))
+                             (return (< (length (car (sb-debug:list-backtrace :count 1)))
+                                        10)))))
        (funcall (compile nil `(lambda (i) (declare ((mod 65536) i)) i)) nil)))))
 
 ;;; bug-1261646
@@ -924,3 +919,172 @@
   5)
 (with-test (:name :clean-fast-method-frame-lossage)
   (low-debug-method 42)) ; no need to assert. it either crashes or doesn't
+
+(defun return-65535 ()
+  65535)
+
+(with-test (:name :indirect-closure-values)
+  (let ((count 0))
+    (block nil
+      (handler-bind ((error (lambda (c)
+                              (declare (ignore c))
+                              (sb-debug::map-backtrace
+                               (lambda (frame)
+                                 (let ((sb-debug::*current-frame* frame)
+                                       (name (sb-debug::frame-call frame)))
+                                   (when (or (eq name 'test)
+                                             (and (consp name)
+                                                  (or (eql (search '(labels f1) name) 0)
+                                                      (eql (search '(labels f2) name) 0))))
+                                     (incf count)
+                                     (assert (eql (var 'a) 2))))))
+                              (return))))
+        (funcall
+         (compile nil
+                  `(sb-int:named-lambda test ()
+                     (declare (optimize debug))
+                     (let ((a 1))
+                       (labels
+                           ((f1 ()
+                              (incf a)
+                              (signal 'error))
+                            (f2 ()
+                              (f1)))
+                         (f2))))))))
+    (assert (= count 3))))
+
+(with-test (:name :indirect-closure-values.2)
+  (let ((count 0))
+    (block nil
+      (handler-bind ((error (lambda (c)
+                              (declare (ignore c))
+                              (sb-debug::map-backtrace
+                               (lambda (frame)
+                                 (let ((sb-debug::*current-frame* frame)
+                                       (name (sb-debug::frame-call frame)))
+                                   (when (or (eq name 'test)
+                                             (and (consp name)
+                                                  (or (eql (search '(labels f1) name) 0)
+                                                      (eql (search '(labels f2) name) 0))))
+                                     (incf count)
+                                     (assert (eql (var 'a) 65535))))))
+                              (return))))
+        (funcall
+         (compile nil
+                  `(sb-int:named-lambda test ()
+                     (declare (optimize debug))
+                     (let ((a (return-65535)))
+                       (declare ((unsigned-byte 16) a))
+                       (labels
+                           ((f1 ()
+                              (incf a)
+                              (signal 'error))
+                            (f2 ()
+                              (f1)))
+                         (f2))))))))
+    (assert (= count 3))))
+
+(with-test (:name :non-tail-self-call-bad-variables)
+  (let ((count 0))
+    (block nil
+      (handler-bind ((error (lambda (c)
+                              (declare (ignore c))
+                              (sb-debug::map-backtrace
+                               (lambda (frame)
+                                 (let ((sb-debug::*current-frame* frame))
+                                   (multiple-value-bind (name args)
+                                       (sb-debug::frame-call frame)
+                                     (when (eq name 'test)
+                                       (assert (or (null args)
+                                                   (equal args '(nil))))
+                                       (incf count))))))
+                              (return))))
+        (funcall
+         (compile nil `(sb-int:named-lambda test (&optional x)
+                         (declare (optimize sb-c::recognize-self-calls))
+                         (signal 'error :format-control "~a" :format-arguments (list x))
+                         (test 1)
+                         1)))))
+    (assert (= count 1))))
+
+(with-test (:name :local-tail-call)
+  (assert (verify-backtrace
+           (lambda () (funcall (compile nil `(sb-int:named-lambda test ()
+                                               (signal 'error)
+                                               (flet ((tail ()))
+                                                 (declare (notinline tail))
+                                                 (tail))))))
+           '((test)))))
+
+(with-test (:name :local-tail-call-variables)
+  (let ((count 0))
+    (block nil
+      (handler-bind ((error (lambda (c)
+                              (declare (ignore c))
+                              (sb-debug::map-backtrace
+                               (lambda (frame)
+                                 (let ((sb-debug::*current-frame* frame))
+                                   (multiple-value-bind (name args)
+                                       (sb-debug::frame-call frame)
+                                     (when (eq name 'test)
+                                       (assert (equal args '(error)))
+                                       (incf count))))))
+                              (return))))
+        (funcall
+         (compile nil `(sb-int:named-lambda test (x)
+                         (signal x)
+                         ;; If :local-tail-call fails, this will fail
+                         ;; too, because there's no jump between
+                         ;; SIGNAL and the call to TAIL and it will
+                         ;; show (flet tail) in the backtrace.
+                         (flet ((tail ()))
+                           (declare (notinline tail))
+                           (tail))))
+         'error)))
+    (assert (= count 1))))
+
+(with-test (:name :variables-surrounding-inlined-code)
+  (let ((count 0))
+    (block nil
+      (handler-bind ((error (lambda (c)
+                              (declare (ignore c))
+                              (sb-debug::map-backtrace
+                               (lambda (frame)
+                                 (let ((sb-debug::*current-frame* frame))
+                                   (multiple-value-bind (name)
+                                       (sb-debug::frame-call frame)
+                                     (when (eq name 'test)
+                                       (assert (equal (sb-debug:var 'l) '(1 2 3)))
+                                       (incf count))))))
+                              (return))))
+        (funcall
+         (compile nil `(sb-int:named-lambda test (a i)
+                         (declare (optimize (debug 3)))
+                         (let ((l (list 1 2 3)))
+                           (aref a i)
+                           l)))
+         #(1) 2)))
+    (assert (= count 1))))
+
+(with-test (:name :variables-surrounding-inlined-code.2)
+  (let ((count 0))
+    (block nil
+      (handler-bind ((error (lambda (c)
+                              (declare (ignore c))
+                              (sb-debug::map-backtrace
+                               (lambda (frame)
+                                 (let ((sb-debug::*current-frame* frame))
+                                   (multiple-value-bind (name)
+                                       (sb-debug::frame-call frame)
+                                     (when (eq name 'test)
+                                       (assert (equal (sb-debug:var 'l) '(1 2 3)))
+                                       (incf count))))))
+                              (return))))
+        (funcall
+         (compile nil `(sb-int:named-lambda test (c)
+                         (declare (optimize (debug 3)))
+                         (let ((l (list 1 2 3)))
+                           (map 'list #'signal c)
+                           l)))
+         '(error))))
+    (assert (= count 1))))
